from __future__ import annotations
import json
from datetime import datetime
from dataclasses import dataclass, field
from tubeulator.utils.string_conv import to_camel_case
from pydantic import AliasGenerator, BaseModel, ConfigDict, PrivateAttr
from ..utils.paths import DtoEnum

class PlaceCategory(BaseModel):
    """
    Autogenerated from StopPoint::Tfl.Api.Presentation.Entities.PlaceCategory
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    Category: str = None
    AvailableKeys: list[str]
    _source_schema_name: str = PrivateAttr(default='StopPoint')
    _component_schema_name: str = PrivateAttr(default='Tfl')


PlaceCategoryModel = PlaceCategory


class Mode(BaseModel):
    """
    Autogenerated from StopPoint::Tfl.Api.Presentation.Entities.Mode
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    IsTflService: bool = None
    IsFarePaying: bool = None
    IsScheduledService: bool = None
    ModeName: str = None
    _source_schema_name: str = PrivateAttr(default='StopPoint')
    _component_schema_name: str = PrivateAttr(default='Tfl-2')


ModeModel = Mode


class PassengerFlow(BaseModel):
    """
    Autogenerated from StopPoint::Tfl.Api.Presentation.Entities.PassengerFlow
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    TimeSlice: str = None
    Value: int = None
    _source_schema_name: str = PrivateAttr(default='StopPoint')
    _component_schema_name: str = PrivateAttr(default='Tfl-3')


PassengerFlowModel = PassengerFlow


class TrainLoading(BaseModel):
    """
    Autogenerated from StopPoint::Tfl.Api.Presentation.Entities.TrainLoading
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    Line: str = None
    LineDirection: str = None
    PlatformDirection: str = None
    Direction: str = None
    NaptanTo: str = None
    TimeSlice: str = None
    Value: int = None
    _source_schema_name: str = PrivateAttr(default='StopPoint')
    _component_schema_name: str = PrivateAttr(default='Tfl-4')


TrainLoadingModel = TrainLoading


class Crowding(BaseModel):
    """
    Autogenerated from StopPoint::Tfl.Api.Presentation.Entities.Crowding
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    PassengerFlows: list["PassengerFlowModel"]
    TrainLoadings: list["TrainLoadingModel"]
    _source_schema_name: str = PrivateAttr(default='StopPoint')
    _component_schema_name: str = PrivateAttr(default='Tfl-5')


CrowdingModel = Crowding


class Identifier(BaseModel):
    """
    Autogenerated from StopPoint::Tfl.Api.Presentation.Entities.Identifier
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    Id: str = None
    Name: str = None
    Uri: str = None
    FullName: str = None
    Type: str = None
    Crowding: CrowdingModel = None
    RouteType: str = None
    Status: str = None
    _source_schema_name: str = PrivateAttr(default='StopPoint')
    _component_schema_name: str = PrivateAttr(default='Tfl-6')


IdentifierModel = Identifier


class LineGroup(BaseModel):
    """
    Autogenerated from StopPoint::Tfl.Api.Presentation.Entities.LineGroup
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    NaptanIdReference: str = None
    StationAtcoCode: str = None
    LineIdentifier: list[str]
    _source_schema_name: str = PrivateAttr(default='StopPoint')
    _component_schema_name: str = PrivateAttr(default='Tfl-7')


LineGroupModel = LineGroup


class LineModeGroup(BaseModel):
    """
    Autogenerated from StopPoint::Tfl.Api.Presentation.Entities.LineModeGroup
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    ModeName: str = None
    LineIdentifier: list[str]
    _source_schema_name: str = PrivateAttr(default='StopPoint')
    _component_schema_name: str = PrivateAttr(default='Tfl-8')


LineModeGroupModel = LineModeGroup


class AdditionalProperties(BaseModel):
    """
    Autogenerated from StopPoint::Tfl.Api.Presentation.Entities.AdditionalProperties
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    Category: str = None
    Key: str = None
    SourceSystemKey: str = None
    Value: str = None
    Modified: datetime = None
    _source_schema_name: str = PrivateAttr(default='StopPoint')
    _component_schema_name: str = PrivateAttr(default='Tfl-9')


AdditionalPropertiesModel = AdditionalProperties


class Place(BaseModel):
    """
    Autogenerated from StopPoint::Tfl.Api.Presentation.Entities.Place
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    Id: str = None
    Url: str = None
    CommonName: str = None
    Distance: float = None
    PlaceType: str = None
    AdditionalProperties: list["AdditionalPropertiesModel"]
    Children: list["PlaceModel"]
    ChildrenUrls: list[str]
    Lat: float = None
    Lon: float = None
    _source_schema_name: str = PrivateAttr(default='StopPoint')
    _component_schema_name: str = PrivateAttr(default='Tfl-10')


PlaceModel = Place


class StopPoint(BaseModel):
    """
    Autogenerated from StopPoint::Tfl.Api.Presentation.Entities.StopPoint
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    NaptanId: str = None
    PlatformName: str = None
    Indicator: str = None
    StopLetter: str = None
    Modes: list[str]
    IcsCode: str = None
    SmsCode: str = None
    StopType: str = None
    StationNaptan: str = None
    AccessibilitySummary: str = None
    HubNaptanCode: str = None
    Lines: list["IdentifierModel"]
    LineGroup: list["LineGroupModel"]
    LineModeGroups: list["LineModeGroupModel"]
    FullName: str = None
    NaptanMode: str = None
    Status: bool = None
    Id: str = None
    Url: str = None
    CommonName: str = None
    Distance: float = None
    PlaceType: str = None
    AdditionalProperties: list["AdditionalPropertiesModel"]
    Children: list["PlaceModel"]
    ChildrenUrls: list[str]
    Lat: float = None
    Lon: float = None
    _source_schema_name: str = PrivateAttr(default='StopPoint')
    _component_schema_name: str = PrivateAttr(default='Tfl-11')


StopPointModel = StopPoint


class LineServiceTypeInfo(BaseModel):
    """
    Autogenerated from StopPoint::Tfl.Api.Presentation.Entities.LineServiceTypeInfo
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    Name: str = None
    Uri: str = None
    _source_schema_name: str = PrivateAttr(default='StopPoint')
    _component_schema_name: str = PrivateAttr(default='Tfl-12')


LineServiceTypeInfoModel = LineServiceTypeInfo


class LineSpecificServiceType(BaseModel):
    """
    Autogenerated from StopPoint::Tfl.Api.Presentation.Entities.LineSpecificServiceType
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    ServiceType: LineServiceTypeInfoModel = None
    StopServesServiceType: bool = None
    _source_schema_name: str = PrivateAttr(default='StopPoint')
    _component_schema_name: str = PrivateAttr(default='Tfl-13')


LineSpecificServiceTypeModel = LineSpecificServiceType


class LineServiceType(BaseModel):
    """
    Autogenerated from StopPoint::Tfl.Api.Presentation.Entities.LineServiceType
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    LineName: str = None
    LineSpecificServiceTypes: list["LineSpecificServiceTypeModel"]
    _source_schema_name: str = PrivateAttr(default='StopPoint')
    _component_schema_name: str = PrivateAttr(default='Tfl-14')


LineServiceTypeModel = LineServiceType


class PredictionTiming(BaseModel):
    """
    Autogenerated from StopPoint::Tfl.Api.Presentation.Entities.PredictionTiming
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    CountdownServerAdjustment: str = None
    Source: datetime = None
    Insert: datetime = None
    Read: datetime = None
    Sent: datetime = None
    Received: datetime = None
    _source_schema_name: str = PrivateAttr(default='StopPoint')
    _component_schema_name: str = PrivateAttr(default='Tfl-15')


PredictionTimingModel = PredictionTiming


class Prediction(BaseModel):
    """
    Autogenerated from StopPoint::Tfl.Api.Presentation.Entities.Prediction
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    Id: str = None
    OperationType: int = None
    VehicleId: str = None
    NaptanId: str = None
    StationName: str = None
    LineId: str = None
    LineName: str = None
    PlatformName: str = None
    Direction: str = None
    Bearing: str = None
    DestinationNaptanId: str = None
    DestinationName: str = None
    Timestamp: datetime = None
    TimeToStation: int = None
    CurrentLocation: str = None
    Towards: str = None
    ExpectedArrival: datetime = None
    TimeToLive: datetime = None
    ModeName: str = None
    Timing: PredictionTimingModel = None
    _source_schema_name: str = PrivateAttr(default='StopPoint')
    _component_schema_name: str = PrivateAttr(default='Tfl-16')


PredictionModel = Prediction


class ArrivalDeparture(BaseModel):
    """
    Autogenerated from StopPoint::Tfl.Api.Presentation.Entities.ArrivalDeparture
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    PlatformName: str = None
    DestinationNaptanId: str = None
    DestinationName: str = None
    NaptanId: str = None
    StationName: str = None
    EstimatedTimeOfArrival: datetime = None
    ScheduledTimeOfArrival: datetime = None
    EstimatedTimeOfDeparture: datetime = None
    ScheduledTimeOfDeparture: datetime = None
    MinutesAndSecondsToArrival: str = None
    MinutesAndSecondsToDeparture: str = None
    Cause: str = None
    DepartureStatus: str = None
    Timing: PredictionTimingModel = None
    _source_schema_name: str = PrivateAttr(default='StopPoint')
    _component_schema_name: str = PrivateAttr(default='Tfl-17')


ArrivalDepartureModel = ArrivalDeparture


class StopPointRouteSection(BaseModel):
    """
    Autogenerated from StopPoint::Tfl.Api.Presentation.Entities.StopPointRouteSection
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    NaptanId: str = None
    LineId: str = None
    Mode: str = None
    ValidFrom: datetime = None
    ValidTo: datetime = None
    Direction: str = None
    RouteSectionName: str = None
    LineString: str = None
    IsActive: bool = None
    ServiceType: str = None
    VehicleDestinationText: str = None
    DestinationName: str = None
    _source_schema_name: str = PrivateAttr(default='StopPoint')
    _component_schema_name: str = PrivateAttr(default='Tfl-18')


StopPointRouteSectionModel = StopPointRouteSection


class DisruptedPoint(BaseModel):
    """
    Autogenerated from StopPoint::Tfl.Api.Presentation.Entities.DisruptedPoint
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    AtcoCode: str = None
    FromDate: datetime = None
    ToDate: datetime = None
    Description: str = None
    CommonName: str = None
    Type: str = None
    Mode: str = None
    StationAtcoCode: str = None
    Appearance: str = None
    AdditionalInformation: str = None
    _source_schema_name: str = PrivateAttr(default='StopPoint')
    _component_schema_name: str = PrivateAttr(default='Tfl-19')


DisruptedPointModel = DisruptedPoint


class StopPointsResponse(BaseModel):
    """
    Autogenerated from StopPoint::Tfl.Api.Presentation.Entities.StopPointsResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    CentrePoint: list[float]
    StopPoints: list["StopPointModel"]
    PageSize: int = None
    Total: int = None
    Page: int = None
    _source_schema_name: str = PrivateAttr(default='StopPoint')
    _component_schema_name: str = PrivateAttr(default='Tfl-20')


StopPointsResponseModel = StopPointsResponse


class SearchMatch(BaseModel):
    """
    Autogenerated from StopPoint::Tfl.Api.Presentation.Entities.SearchMatch
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    Id: str = None
    Url: str = None
    Name: str = None
    Lat: float = None
    Lon: float = None
    _source_schema_name: str = PrivateAttr(default='StopPoint')
    _component_schema_name: str = PrivateAttr(default='Tfl-21')


SearchMatchModel = SearchMatch


class SearchResponse(BaseModel):
    """
    Autogenerated from StopPoint::Tfl.Api.Presentation.Entities.SearchResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    Query: str = None
    From: int = None
    Page: int = None
    PageSize: int = None
    Provider: str = None
    Total: int = None
    Matches: list["SearchMatchModel"]
    MaxScore: float = None
    _source_schema_name: str = PrivateAttr(default='StopPoint')
    _component_schema_name: str = PrivateAttr(default='Tfl-22')


SearchResponseModel = SearchResponse


class Object(BaseModel):
    """
    Autogenerated from StopPoint::System.Object
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='StopPoint')
    _component_schema_name: str = PrivateAttr(default='System')


ObjectModel = Object


class Get200ApplicationJsonResponse(BaseModel):
    """
    Autogenerated from StopPoint::Get200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='StopPoint')
    _component_schema_name: str = PrivateAttr(default='Get200ApplicationJsonResponse')


Get200ApplicationJsonResponseModel = Get200ApplicationJsonResponse


class MetaCategoriesGet200ApplicationJsonResponse(BaseModel):
    """
    Autogenerated from StopPoint::MetaCategoriesGet200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='StopPoint')
    _component_schema_name: str = PrivateAttr(default='MetaCategoriesGet200ApplicationJsonResponse')


MetaCategoriesGet200ApplicationJsonResponseModel = MetaCategoriesGet200ApplicationJsonResponse


class MetaStopTypes(BaseModel):
    """
    Autogenerated from StopPoint::MetaStopTypesGet200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='StopPoint')
    _component_schema_name: str = PrivateAttr(default='MetaStopTypesGet200ApplicationJsonResponse')


MetaStopTypesModel = MetaStopTypes


class MetaModesGet200ApplicationJsonResponse(BaseModel):
    """
    Autogenerated from StopPoint::MetaModesGet200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='StopPoint')
    _component_schema_name: str = PrivateAttr(default='MetaModesGet200ApplicationJsonResponse')


MetaModesGet200ApplicationJsonResponseModel = MetaModesGet200ApplicationJsonResponse


class idsGet200ApplicationJsonResponse(BaseModel):
    """
    Autogenerated from StopPoint::ids-Get200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='StopPoint')
    _component_schema_name: str = PrivateAttr(default='ids-Get200ApplicationJsonResponse')


idsGet200ApplicationJsonResponseModel = idsGet200ApplicationJsonResponse


class idPlaceTypesGet200ApplicationJsonResponse(BaseModel):
    """
    Autogenerated from StopPoint::id-PlaceTypesGet200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='StopPoint')
    _component_schema_name: str = PrivateAttr(default='id-PlaceTypesGet200ApplicationJsonResponse')


idPlaceTypesGet200ApplicationJsonResponseModel = idPlaceTypesGet200ApplicationJsonResponse


class idCrowdinglineGet200ApplicationJsonResponse(BaseModel):
    """
    Autogenerated from StopPoint::id-Crowding-line-Get200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='StopPoint')
    _component_schema_name: str = PrivateAttr(default='id-Crowding-line-Get200ApplicationJsonResponse')


idCrowdinglineGet200ApplicationJsonResponseModel = idCrowdinglineGet200ApplicationJsonResponse


class TypetypesGet200ApplicationJsonResponse(BaseModel):
    """
    Autogenerated from StopPoint::Type-types-Get200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='StopPoint')
    _component_schema_name: str = PrivateAttr(default='Type-types-Get200ApplicationJsonResponse')


TypetypesGet200ApplicationJsonResponseModel = TypetypesGet200ApplicationJsonResponse


class TypetypesPagepageGet200ApplicationJsonResponse(BaseModel):
    """
    Autogenerated from StopPoint::Type-types-Page-page-Get200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='StopPoint')
    _component_schema_name: str = PrivateAttr(default='Type-types-Page-page-Get200ApplicationJsonResponse')


TypetypesPagepageGet200ApplicationJsonResponseModel = TypetypesPagepageGet200ApplicationJsonResponse


class ServiceTypesGet200ApplicationJsonResponse(BaseModel):
    """
    Autogenerated from StopPoint::ServiceTypesGet200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='StopPoint')
    _component_schema_name: str = PrivateAttr(default='ServiceTypesGet200ApplicationJsonResponse')


ServiceTypesGet200ApplicationJsonResponseModel = ServiceTypesGet200ApplicationJsonResponse


class idArrivalsGet200ApplicationJsonResponse(BaseModel):
    """
    Autogenerated from StopPoint::id-ArrivalsGet200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='StopPoint')
    _component_schema_name: str = PrivateAttr(default='id-ArrivalsGet200ApplicationJsonResponse')


idArrivalsGet200ApplicationJsonResponseModel = idArrivalsGet200ApplicationJsonResponse


class idArrivalDeparturesGet200ApplicationJsonResponse(BaseModel):
    """
    Autogenerated from StopPoint::id-ArrivalDeparturesGet200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='StopPoint')
    _component_schema_name: str = PrivateAttr(default='id-ArrivalDeparturesGet200ApplicationJsonResponse')


idArrivalDeparturesGet200ApplicationJsonResponseModel = idArrivalDeparturesGet200ApplicationJsonResponse


class idCanReachOnLinelineIdGet200ApplicationJsonResponse(BaseModel):
    """
    Autogenerated from StopPoint::id-CanReachOnLine-lineId-Get200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='StopPoint')
    _component_schema_name: str = PrivateAttr(default='id-CanReachOnLine-lineId-Get200ApplicationJsonResponse')


idCanReachOnLinelineIdGet200ApplicationJsonResponseModel = idCanReachOnLinelineIdGet200ApplicationJsonResponse


class idRouteGet200ApplicationJsonResponse(BaseModel):
    """
    Autogenerated from StopPoint::id-RouteGet200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='StopPoint')
    _component_schema_name: str = PrivateAttr(default='id-RouteGet200ApplicationJsonResponse')


idRouteGet200ApplicationJsonResponseModel = idRouteGet200ApplicationJsonResponse


class ModemodesDisruptionGet200ApplicationJsonResponse(BaseModel):
    """
    Autogenerated from StopPoint::Mode-modes-DisruptionGet200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='StopPoint')
    _component_schema_name: str = PrivateAttr(default='Mode-modes-DisruptionGet200ApplicationJsonResponse')


ModemodesDisruptionGet200ApplicationJsonResponseModel = ModemodesDisruptionGet200ApplicationJsonResponse


class idsDisruptionGet200ApplicationJsonResponse(BaseModel):
    """
    Autogenerated from StopPoint::ids-DisruptionGet200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='StopPoint')
    _component_schema_name: str = PrivateAttr(default='ids-DisruptionGet200ApplicationJsonResponse')


idsDisruptionGet200ApplicationJsonResponseModel = idsDisruptionGet200ApplicationJsonResponse


class idDirectionTotoStopPointIdGet200ApplicationJsonResponse(BaseModel):
    """
    Autogenerated from StopPoint::id-DirectionTo-toStopPointId-Get200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='StopPoint')
    _component_schema_name: str = PrivateAttr(default='id-DirectionTo-toStopPointId-Get200ApplicationJsonResponse')


idDirectionTotoStopPointIdGet200ApplicationJsonResponseModel = idDirectionTotoStopPointIdGet200ApplicationJsonResponse


class stopPointIdTaxiRanksGet200ApplicationJsonResponse(BaseModel):
    """
    Autogenerated from StopPoint::stopPointId-TaxiRanksGet200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='StopPoint')
    _component_schema_name: str = PrivateAttr(default='stopPointId-TaxiRanksGet200ApplicationJsonResponse')


stopPointIdTaxiRanksGet200ApplicationJsonResponseModel = stopPointIdTaxiRanksGet200ApplicationJsonResponse


class stopPointIdCarParksGet200ApplicationJsonResponse(BaseModel):
    """
    Autogenerated from StopPoint::stopPointId-CarParksGet200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='StopPoint')
    _component_schema_name: str = PrivateAttr(default='stopPointId-CarParksGet200ApplicationJsonResponse')


stopPointIdCarParksGet200ApplicationJsonResponseModel = stopPointIdCarParksGet200ApplicationJsonResponse


class Deserialisers(DtoEnum):
    Tfl = PlaceCategory
    Tfl_2 = Mode
    Tfl_3 = PassengerFlow
    Tfl_4 = TrainLoading
    Tfl_5 = Crowding
    Tfl_6 = Identifier
    Tfl_7 = LineGroup
    Tfl_8 = LineModeGroup
    Tfl_9 = AdditionalProperties
    Tfl_10 = Place
    Tfl_11 = StopPoint
    Tfl_12 = LineServiceTypeInfo
    Tfl_13 = LineSpecificServiceType
    Tfl_14 = LineServiceType
    Tfl_15 = PredictionTiming
    Tfl_16 = Prediction
    Tfl_17 = ArrivalDeparture
    Tfl_18 = StopPointRouteSection
    Tfl_19 = DisruptedPoint
    Tfl_20 = StopPointsResponse
    Tfl_21 = SearchMatch
    Tfl_22 = SearchResponse
    System = Object
    Get200ApplicationJsonResponse = Get200ApplicationJsonResponse
    MetaCategoriesGet200ApplicationJsonResponse = MetaCategoriesGet200ApplicationJsonResponse
    MetaStopTypesGet200ApplicationJsonResponse = MetaStopTypes
    MetaModesGet200ApplicationJsonResponse = MetaModesGet200ApplicationJsonResponse
    ids_Get200ApplicationJsonResponse = idsGet200ApplicationJsonResponse
    id_PlaceTypesGet200ApplicationJsonResponse = idPlaceTypesGet200ApplicationJsonResponse
    id_Crowding_line_Get200ApplicationJsonResponse = idCrowdinglineGet200ApplicationJsonResponse
    Type_types_Get200ApplicationJsonResponse = TypetypesGet200ApplicationJsonResponse
    Type_types_Page_page_Get200ApplicationJsonResponse = TypetypesPagepageGet200ApplicationJsonResponse
    ServiceTypesGet200ApplicationJsonResponse = ServiceTypesGet200ApplicationJsonResponse
    id_ArrivalsGet200ApplicationJsonResponse = idArrivalsGet200ApplicationJsonResponse
    id_ArrivalDeparturesGet200ApplicationJsonResponse = idArrivalDeparturesGet200ApplicationJsonResponse
    id_CanReachOnLine_lineId_Get200ApplicationJsonResponse = idCanReachOnLinelineIdGet200ApplicationJsonResponse
    id_RouteGet200ApplicationJsonResponse = idRouteGet200ApplicationJsonResponse
    Mode_modes_DisruptionGet200ApplicationJsonResponse = ModemodesDisruptionGet200ApplicationJsonResponse
    ids_DisruptionGet200ApplicationJsonResponse = idsDisruptionGet200ApplicationJsonResponse
    id_DirectionTo_toStopPointId_Get200ApplicationJsonResponse = idDirectionTotoStopPointIdGet200ApplicationJsonResponse
    stopPointId_TaxiRanksGet200ApplicationJsonResponse = stopPointIdTaxiRanksGet200ApplicationJsonResponse
    stopPointId_CarParksGet200ApplicationJsonResponse = stopPointIdCarParksGet200ApplicationJsonResponse


from .types import RouteEnum


class RoadEndpointRoutes(RouteEnum):
    ALL_ROADS = "/"
    """Gets all roads managed by TfL"""
    META_CATEGORIES = "/Meta/Categories"
    """Gets a list of valid RoadDisruption categories"""
    META_SEVERITIES = "/Meta/Severities"
    """Gets a list of valid RoadDisruption severity codes"""
    DISRUPTION_BY_IDS = "/all/Disruption/{disruptionIds}"
    """Gets a list of active disruptions filtered by disruption Ids."""
    STREET_DISRUPTION = "/all/Street/Disruption"
    """Gets a list of disrupted streets. If no date filters are provided, current
    disruptions are returned."""
    ROAD_BY_IDS = "/{ids}"
    """Gets the road with the specified id (e.g. A1)"""
    ROAD_DISRUPTION = "/{ids}/Disruption"
    """Get active disruptions, filtered by road ids"""
    ROAD_STATUS = "/{ids}/Status"
    """Gets the specified roads with the status aggregated over the date range
    specified, or now until the end of today if no dates are passed."""



from __future__ import annotations

from functools import lru_cache
from pathlib import Path

from ..utils.paths import find_schema, openapi_schemas_path
from ._types import ApiAliasToUnifiedEntities, NamespaceInventory
from .schema.aliased import AliasedApiSchema

__all__ = ["api_schema_inventory", "scan_namespace"]


# TODO: duplication here suggests need for an API schema class (from JSON to entities)
def api_schema_inventory(api_schema: Path) -> ApiAliasToUnifiedEntities:
    """Make a mapping of all aliases (the arbitrarily named entities) to their (properly
    named) unified entity names.
    """
    aliased_api_schema = AliasedApiSchema(path=api_schema, unified=False)
    # aliased_api_schema.pprint_api_inventory()
    return aliased_api_schema.alias2ents


@lru_cache
def scan_namespace(ignore_responses: bool = False) -> NamespaceInventory:
    api_dirs = [d for d in openapi_schemas_path.iterdir() if d.is_dir()]
    api_schemas = [
        find_schema(schema_dir=api_dir, match=api_dir.stem) for api_dir in api_dirs
    ]
    namespace_inventory = {
        api_schema.stem: {
            alias: entity
            for alias, entity in api_schema_inventory(api_schema).items()
            if "Response" not in alias
            # Responses are v degenerate and make the output unreadable
        }
        for api_schema in api_schemas
    }
    return namespace_inventory


def count_namespace(ignore_responses: bool = False) -> NamespaceInventory:
    ns_inventory = scan_namespace(ignore_responses=ignore_responses)
    ns_counts = {
        schema: {
            "pass": len([v for v in ns_inventory[schema].values() if v]),
            "fail": len([v for v in ns_inventory[schema].values() if not v]),
        }
        for schema in ns_inventory
    }
    ns_summaries = {
        schema: f"{ns_counts[schema]['pass']} of {len(ns_inventory[schema])}"
        for schema in ns_inventory
    }
    return ns_summaries



from __future__ import annotations

from copy import deepcopy
from dataclasses import dataclass
from functools import reduce
from pathlib import Path

from ._types import ApiAliasToUnifiedEntities, ApiEntityAlias, EntityURI

__all__ = [
    "Reference",
    "AliasToRefs",
    "dealias_schema",
]


@dataclass
class Reference:
    schema: str
    alias: str
    referential_property: str
    # path: tuple[Literal["items","$ref"]]
    is_array: bool
    entity: EntityURI

    @property
    def path(self) -> tuple[str, ...]:
        return (*(["items"] if self.is_array else []), "$ref")

    @property
    def substituted_ref(self) -> str:
        return Path(self.entity).name

    def is_completable(self, api_inventory: ApiAliasToUnifiedEntities):
        """If the API inventory has a resolved value (or values) for the alias referenced
        in the current referential property's ``$ref`` key, then the reference can be
        resolved immediately (i.e. "completed"). If not, we have to wait for the next
        round of substitutions, or chase references.
        """
        if self.substituted_ref in api_inventory:
            return api_inventory[self.substituted_ref] != []
        else:
            raise ValueError(f"{self.substituted_ref=} not found in {api_inventory=}")


AliasToRefs = dict[ApiEntityAlias, list[Reference]]
"""
An inventory of API entity aliases and the references they contain.
"""


def dealias_schema(refs: list[Reference], component: dict, copy: bool = True) -> dict:
    """Modify the schema component of an API schema to overwrite the reference with
    the entity name in the :obj:`Reference` instead of the alias there.
    """
    if copy:
        component = deepcopy(component)
    for ref in refs:
        target_property = component["properties"][ref.referential_property]
        target_subpath = ref.path[:-1]
        target_leaf = ref.path[-1]
        target = reduce(dict.get, target_subpath, target_property)
        target[target_leaf] = ref.entity
    return component



from __future__ import annotations
import json
from datetime import datetime
from dataclasses import dataclass, field
from tubeulator.utils.string_conv import to_camel_case
from pydantic import AliasGenerator, BaseModel, ConfigDict, PrivateAttr
from ..utils.paths import DtoEnum

class PlaceArray(BaseModel):
    """
    Autogenerated from BikePoint::Tfl-Api-Presentation-Entities-PlaceArray
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='BikePoint')
    _component_schema_name: str = PrivateAttr(default='Tfl-Api-Presentation-Entities-PlaceArray')


PlaceArrayModel = PlaceArray


class Place(BaseModel):
    """
    Autogenerated from BikePoint::Tfl.Api.Presentation.Entities.Place
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    Id: str = None
    Url: str = None
    CommonName: str = None
    Distance: float = None
    PlaceType: str = None
    AdditionalProperties: list["AdditionalPropertiesModel"]
    Children: list["PlaceModel"]
    ChildrenUrls: list[str]
    Lat: float = None
    Lon: float = None
    _source_schema_name: str = PrivateAttr(default='BikePoint')
    _component_schema_name: str = PrivateAttr(default='Tfl.Api.Presentation.Entities.Place')


PlaceModel = Place


class AdditionalProperties(BaseModel):
    """
    Autogenerated from BikePoint::Tfl.Api.Presentation.Entities.AdditionalProperties
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    Category: str = None
    Key: str = None
    SourceSystemKey: str = None
    Value: str = None
    Modified: datetime = None
    _source_schema_name: str = PrivateAttr(default='BikePoint')
    _component_schema_name: str = PrivateAttr(default='Tfl.Api.Presentation.Entities.AdditionalProperties')


AdditionalPropertiesModel = AdditionalProperties


class Deserialisers(DtoEnum):
    Tfl_Api_Presentation_Entities_PlaceArray = PlaceArray
    Tfl__Api__Presentation__Entities__Place = Place
    Tfl__Api__Presentation__Entities__AdditionalProperties = AdditionalProperties


from .endpoint.names import EndpointNames
from .endpoint.routes import EndpointRoute
from .endpoint.routes.types import AnyEndpointRouteEnum
from .request import Path, Request

__all__ = ["EndpointInfo", "AllEndpointInterface", "fetch"]


class EndpointInfo:
    def __init__(self, endpoint: AnyEndpointRouteEnum):
        for route in endpoint.value:
            endpoint_name = EndpointNames[endpoint.name]
            request = Request(path=Path(endpoint=endpoint_name, route=route))
            setattr(self, route.name.lower(), request)


class AllEndpointInterface:
    def __init__(self):
        for endpoint in EndpointRoute:
            interface = EndpointInfo(endpoint=endpoint)
            setattr(self, endpoint.name.lower(), interface)


fetch = AllEndpointInterface()



from .types import RouteEnum


class SearchEndpointRoutes(RouteEnum):
    SEARCH = "/"
    """Search the site for occurrences of the query string. The maximum number of
    results returned is equal to the maximum page size of 100. To return subsequent
    pages, use the paginated overload."""
    BUS_SCHEDULES = "/BusSchedules"
    """Searches the bus schedules folder on S3 for a given bus number."""
    META_CATEGORIES = "/Meta/Categories"
    """Gets the available search categories."""
    META_SEARCH_PROVIDERS = "/Meta/SearchProviders"
    """Gets the available searchProvider names."""
    META_SORTS = "/Meta/Sorts"
    """Gets the available sorting options."""



from contextlib import suppress
from pathlib import Path

from httpx import Response

__all__ = ["RequestError"]


class RequestError(Exception):
    def __init__(
        self,
        response: Response,
        path: Path,
        _args: tuple[str],
        _kwargs: dict[str, str],
    ):
        path_repr = f"{path.endpoint.name}.{path.route.name}:{path.route.value}"
        reason = f"{response.status_code} {response.reason_phrase}"
        message = f"HTTP error at {path_repr} ({list(_args)}, {_kwargs})\n  {reason}"
        with suppress(Exception):
            message += f": {response.json()['message']}"
        self.message = message
        self.response = response
        super().__init__(self.message)



from __future__ import annotations
import json
from datetime import datetime
from dataclasses import dataclass, field
from tubeulator.utils.string_conv import to_camel_case
from pydantic import AliasGenerator, BaseModel, ConfigDict, PrivateAttr
from ..utils.paths import DtoEnum

class PlaceCategory(BaseModel):
    """
    Autogenerated from Place::Tfl.Api.Presentation.Entities.PlaceCategory
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    Category: str = None
    AvailableKeys: list[str]
    _source_schema_name: str = PrivateAttr(default='Place')
    _component_schema_name: str = PrivateAttr(default='Tfl')


PlaceCategoryModel = PlaceCategory


class AdditionalProperties(BaseModel):
    """
    Autogenerated from Place::Tfl.Api.Presentation.Entities.AdditionalProperties
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    Category: str = None
    Key: str = None
    SourceSystemKey: str = None
    Value: str = None
    Modified: datetime = None
    _source_schema_name: str = PrivateAttr(default='Place')
    _component_schema_name: str = PrivateAttr(default='Tfl-2')


AdditionalPropertiesModel = AdditionalProperties


class Place(BaseModel):
    """
    Autogenerated from Place::Tfl.Api.Presentation.Entities.Place
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    Id: str = None
    Url: str = None
    CommonName: str = None
    Distance: float = None
    PlaceType: str = None
    AdditionalProperties: list["AdditionalPropertiesModel"]
    Children: list["PlaceModel"]
    ChildrenUrls: list[str]
    Lat: float = None
    Lon: float = None
    _source_schema_name: str = PrivateAttr(default='Place')
    _component_schema_name: str = PrivateAttr(default='Tfl-3')


PlaceModel = Place


class PassengerFlow(BaseModel):
    """
    Autogenerated from Place::Tfl.Api.Presentation.Entities.PassengerFlow
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    TimeSlice: str = None
    Value: int = None
    _source_schema_name: str = PrivateAttr(default='Place')
    _component_schema_name: str = PrivateAttr(default='Tfl-4')


PassengerFlowModel = PassengerFlow


class TrainLoading(BaseModel):
    """
    Autogenerated from Place::Tfl.Api.Presentation.Entities.TrainLoading
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    Line: str = None
    LineDirection: str = None
    PlatformDirection: str = None
    Direction: str = None
    NaptanTo: str = None
    TimeSlice: str = None
    Value: int = None
    _source_schema_name: str = PrivateAttr(default='Place')
    _component_schema_name: str = PrivateAttr(default='Tfl-5')


TrainLoadingModel = TrainLoading


class Crowding(BaseModel):
    """
    Autogenerated from Place::Tfl.Api.Presentation.Entities.Crowding
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    PassengerFlows: list["PassengerFlowModel"]
    TrainLoadings: list["TrainLoadingModel"]
    _source_schema_name: str = PrivateAttr(default='Place')
    _component_schema_name: str = PrivateAttr(default='Tfl-6')


CrowdingModel = Crowding


class Identifier(BaseModel):
    """
    Autogenerated from Place::Tfl.Api.Presentation.Entities.Identifier
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    Id: str = None
    Name: str = None
    Uri: str = None
    FullName: str = None
    Type: str = None
    Crowding: CrowdingModel = None
    RouteType: str = None
    Status: str = None
    _source_schema_name: str = PrivateAttr(default='Place')
    _component_schema_name: str = PrivateAttr(default='Tfl-7')


IdentifierModel = Identifier


class LineGroup(BaseModel):
    """
    Autogenerated from Place::Tfl.Api.Presentation.Entities.LineGroup
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    NaptanIdReference: str = None
    StationAtcoCode: str = None
    LineIdentifier: list[str]
    _source_schema_name: str = PrivateAttr(default='Place')
    _component_schema_name: str = PrivateAttr(default='Tfl-8')


LineGroupModel = LineGroup


class LineModeGroup(BaseModel):
    """
    Autogenerated from Place::Tfl.Api.Presentation.Entities.LineModeGroup
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    ModeName: str = None
    LineIdentifier: list[str]
    _source_schema_name: str = PrivateAttr(default='Place')
    _component_schema_name: str = PrivateAttr(default='Tfl-9')


LineModeGroupModel = LineModeGroup


class StopPoint(BaseModel):
    """
    Autogenerated from Place::Tfl.Api.Presentation.Entities.StopPoint
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    NaptanId: str = None
    PlatformName: str = None
    Indicator: str = None
    StopLetter: str = None
    Modes: list[str]
    IcsCode: str = None
    SmsCode: str = None
    StopType: str = None
    StationNaptan: str = None
    AccessibilitySummary: str = None
    HubNaptanCode: str = None
    Lines: list["IdentifierModel"]
    LineGroup: list["LineGroupModel"]
    LineModeGroups: list["LineModeGroupModel"]
    FullName: str = None
    NaptanMode: str = None
    Status: bool = None
    Id: str = None
    Url: str = None
    CommonName: str = None
    Distance: float = None
    PlaceType: str = None
    AdditionalProperties: list["AdditionalPropertiesModel"]
    Children: list["PlaceModel"]
    ChildrenUrls: list[str]
    Lat: float = None
    Lon: float = None
    _source_schema_name: str = PrivateAttr(default='Place')
    _component_schema_name: str = PrivateAttr(default='Tfl-10')


StopPointModel = StopPoint


class Object(BaseModel):
    """
    Autogenerated from Place::System.Object
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='Place')
    _component_schema_name: str = PrivateAttr(default='System')


ObjectModel = Object


class MetaCategoriesGet200ApplicationJsonResponse(BaseModel):
    """
    Autogenerated from Place::MetaCategoriesGet200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='Place')
    _component_schema_name: str = PrivateAttr(default='MetaCategoriesGet200ApplicationJsonResponse')


MetaCategoriesGet200ApplicationJsonResponseModel = MetaCategoriesGet200ApplicationJsonResponse


class Get200ApplicationJsonResponse(BaseModel):
    """
    Autogenerated from Place::Get200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='Place')
    _component_schema_name: str = PrivateAttr(default='Get200ApplicationJsonResponse')


Get200ApplicationJsonResponseModel = Get200ApplicationJsonResponse


class MetaPlaceTypesGet200ApplicationJsonResponse(BaseModel):
    """
    Autogenerated from Place::MetaPlaceTypesGet200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='Place')
    _component_schema_name: str = PrivateAttr(default='MetaPlaceTypesGet200ApplicationJsonResponse')


MetaPlaceTypesGet200ApplicationJsonResponseModel = MetaPlaceTypesGet200ApplicationJsonResponse


class TypetypesGet200ApplicationJsonResponse(BaseModel):
    """
    Autogenerated from Place::Type-types-Get200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='Place')
    _component_schema_name: str = PrivateAttr(default='Type-types-Get200ApplicationJsonResponse')


TypetypesGet200ApplicationJsonResponseModel = TypetypesGet200ApplicationJsonResponse


class idGet200ApplicationJsonResponse(BaseModel):
    """
    Autogenerated from Place::id-Get200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='Place')
    _component_schema_name: str = PrivateAttr(default='id-Get200ApplicationJsonResponse')


idGet200ApplicationJsonResponseModel = idGet200ApplicationJsonResponse


class ApplicationJsonResponse(BaseModel):
    """
    Autogenerated from Place::Get200ApplicationJsonResponse-1
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='Place')
    _component_schema_name: str = PrivateAttr(default='Get200ApplicationJsonResponse-1')


ApplicationJsonResponseModel = ApplicationJsonResponse


class SearchGet200ApplicationJsonResponse(BaseModel):
    """
    Autogenerated from Place::SearchGet200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='Place')
    _component_schema_name: str = PrivateAttr(default='SearchGet200ApplicationJsonResponse')


SearchGet200ApplicationJsonResponseModel = SearchGet200ApplicationJsonResponse


class Deserialisers(DtoEnum):
    Tfl = PlaceCategory
    Tfl_2 = AdditionalProperties
    Tfl_3 = Place
    Tfl_4 = PassengerFlow
    Tfl_5 = TrainLoading
    Tfl_6 = Crowding
    Tfl_7 = Identifier
    Tfl_8 = LineGroup
    Tfl_9 = LineModeGroup
    Tfl_10 = StopPoint
    System = Object
    MetaCategoriesGet200ApplicationJsonResponse = MetaCategoriesGet200ApplicationJsonResponse
    Get200ApplicationJsonResponse = Get200ApplicationJsonResponse
    MetaPlaceTypesGet200ApplicationJsonResponse = MetaPlaceTypesGet200ApplicationJsonResponse
    Type_types_Get200ApplicationJsonResponse = TypetypesGet200ApplicationJsonResponse
    id_Get200ApplicationJsonResponse = idGet200ApplicationJsonResponse
    Get200ApplicationJsonResponse_1 = ApplicationJsonResponse
    SearchGet200ApplicationJsonResponse = SearchGet200ApplicationJsonResponse


"""Helpers to process the JSONSchema format (a.k.a. Swagger)."""

__all__ = ["python_type"]


def python_type(json_type: str, format: str = None) -> str:
    """Map property types in the JSON schema to Python types for type annotation.

    The `format` can be "email" if it's a string but it doesn't change the result.
    """
    type_lookup = {
        "string": "str",
        "integer": "int",
        "boolean": "bool",
        "array": "list",
        "object": "dict",
    }
    if json_type == "number" and format == "double":
        python_type = "float"
    elif json_type == "string" and format == "date-time":
        python_type = "datetime"
    else:
        python_type = type_lookup[json_type]
    return python_type



"""Emit DTO code generated for TfL APIs (detected from their loaded Swagger schemas)."""
from ...openapi.scan import scan_namespace
from ...utils.paths import load_endpoint_component_schemas, to_enum_friendly_str
from .dto import generate_dataclass, generate_dataclass_name
from .helpers import logger

__all__ = ["emit_deserialisers"]


def emit_deserialisers(schema_name: str) -> str:
    """Generate the source code for a module with DTOs for an entire TfL schema."""
    component_schemas = load_endpoint_component_schemas(schema_name)
    ns = scan_namespace(ignore_responses=True)
    output = []
    schema_mapping = {}
    # 1st pass for the generated dataclass names (populating the schema mapping)
    for idx, (component_name, component_schema) in enumerate(component_schemas.items()):
        true_name = (ns.get(schema_name, {}).get(component_name) or [None])[0]
        try:
            generated_class_name = generate_dataclass_name(
                component_schema,
                name=component_name,
                source_schema_name=schema_name,
                idx=idx,
                dealiased_name=true_name,
            )
            if generated_class_name is not None:
                schema_mapping.setdefault(component_name, generated_class_name)
        except Exception as e:
            logger.error(
                f"Failed to generate name for {schema_name} dataclass {idx}: "
                f"{component_name} (dealiased: {true_name}) -- {e}",
                exc_info=True,
            )
    # 2nd pass for the source code (using the schema mapping)
    for idx, (component_name, component_schema) in enumerate(component_schemas.items()):
        true_name = (ns.get(schema_name, {}).get(component_name) or [None])[0]
        try:
            generated_class_name, generated_code = generate_dataclass(
                component_schema,
                name=component_name,
                source_schema_name=schema_name,
                idx=idx,
                dealiased_name=true_name,
                schema_mapping=schema_mapping,
            )
            output.append(generated_code)  # Append even if None, for debugging purposes
        except Exception as e:
            logger.error(
                f"Failed to generate {schema_name} dataclass {idx}: "
                f"{component_name} (dealiased: {true_name}) -- {e}",
                exc_info=True,
            )
    deserialiser_code = "\n".join(filter(None, output))
    if deserialiser_code:
        schema_mapping_source = "\n\n\nclass Deserialisers(DtoEnum):\n" + "\n".join(
            [
                f"    {to_enum_friendly_str(component_name)} = {cls_name}"
                for component_name, cls_name in schema_mapping.items()
            ],
        )
        deserialiser_code += schema_mapping_source
    return deserialiser_code



from __future__ import annotations
import json
from datetime import datetime
from dataclasses import dataclass, field
from tubeulator.utils.string_conv import to_camel_case
from pydantic import AliasGenerator, BaseModel, ConfigDict, PrivateAttr
from ..utils.paths import DtoEnum

class SearchMatch(BaseModel):
    """
    Autogenerated from Search::Tfl.Api.Presentation.Entities.SearchMatch
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    Id: str = None
    Url: str = None
    Name: str = None
    Lat: float = None
    Lon: float = None
    _source_schema_name: str = PrivateAttr(default='Search')
    _component_schema_name: str = PrivateAttr(default='Tfl')


SearchMatchModel = SearchMatch


class SearchResponse(BaseModel):
    """
    Autogenerated from Search::Tfl.Api.Presentation.Entities.SearchResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    Query: str = None
    From: int = None
    Page: int = None
    PageSize: int = None
    Provider: str = None
    Total: int = None
    Matches: list["SearchMatchModel"]
    MaxScore: float = None
    _source_schema_name: str = PrivateAttr(default='Search')
    _component_schema_name: str = PrivateAttr(default='Tfl-2')


SearchResponseModel = SearchResponse


class MetaSearchProviders(BaseModel):
    """
    Autogenerated from Search::MetaSearchProvidersGet200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='Search')
    _component_schema_name: str = PrivateAttr(default='MetaSearchProvidersGet200ApplicationJsonResponse')


MetaSearchProvidersModel = MetaSearchProviders


class MetaCategories(BaseModel):
    """
    Autogenerated from Search::MetaCategoriesGet200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='Search')
    _component_schema_name: str = PrivateAttr(default='MetaCategoriesGet200ApplicationJsonResponse')


MetaCategoriesModel = MetaCategories


class MetaSorts(BaseModel):
    """
    Autogenerated from Search::MetaSortsGet200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='Search')
    _component_schema_name: str = PrivateAttr(default='MetaSortsGet200ApplicationJsonResponse')


MetaSortsModel = MetaSorts


class Deserialisers(DtoEnum):
    Tfl = SearchMatch
    Tfl_2 = SearchResponse
    MetaSearchProvidersGet200ApplicationJsonResponse = MetaSearchProviders
    MetaCategoriesGet200ApplicationJsonResponse = MetaCategories
    MetaSortsGet200ApplicationJsonResponse = MetaSorts


"""LCP trie adapted from dict (char) trie at:
https://ychai.uk/notes/2019/03/03/Programming/Tricks-of-Python/
"""
from __future__ import annotations

__all__ = ["TrieNode", "Trie"]


class TrieNode:
    def __init__(self):
        self.data = {}
        self.is_word = False


class Trie:
    def __init__(self):
        self.root = TrieNode()

    def insert(self, word: str) -> None:
        """Inserts a word into the trie and aggregates it with existing nodes (longest
        common prefix nodes).
        """
        node = self.root
        # If no common ancestors, insert entire word
        if not node.data:
            node.data[word] = TrieNode()
            node.data[word].is_word = True
            return
        # Iterate through existing nodes
        while True:
            # Find the longest common prefix between the word and node keys
            lcp = ""
            lcp_key = None
            for key in node.data.keys():
                prefix = ""
                for i, (a, b) in enumerate(zip(word, key)):
                    if a == b:
                        prefix += a
                    else:
                        break
                if len(prefix) > len(lcp):
                    lcp = prefix
                    lcp_key = key
            if not lcp:
                # No common prefix, insert entire word as new node
                node.data[word] = TrieNode()
                node.data[word].is_word = True
                return
            # If entire common ancestor is found, insert below
            lcp_size = len(lcp)
            if lcp == lcp_key:
                node = node.data[lcp_key]
                word = word[lcp_size:]
            else:
                # Split the node at the branch point and insert the rest of the word below it
                new_node = TrieNode()
                node.data[lcp] = new_node
                new_node.data[lcp_key[lcp_size:]] = node.data.pop(lcp_key)
                if not word:
                    # The branch point is the word
                    new_node.is_word = True
                else:
                    # Put the remaining suffix below the LCP branch point node
                    partial_word = word[lcp_size:]
                    new_node.data[partial_word] = TrieNode()
                    new_node.data[partial_word].is_word = True
                return

    def search(self, word: str) -> None:
        """Returns if the word is in the trie."""
        node = self.root
        for letter in word:
            node = node.data.get(letter)
            if not node:
                return False
        return node.is_word

    def starts_with(self, prefix: str) -> bool:
        """Returns if there is any word in the trie
        that starts with the given prefix.
        """
        node = self.root
        for letter in prefix:
            node = node.data.get(letter)
            if not node:
                return False
        return True

    def get_start(self, prefix: str) -> list[str]:
        """Returns words started with prefix"""

        def _get_key(pre, pre_node):
            words_list = []
            if pre_node.is_word:
                words_list.append(pre)
            for x in pre_node.data.keys():
                words_list.extend(_get_key(pre + str(x), pre_node.data.get(x)))
            return words_list

        words = []
        if not self.starts_with(prefix):
            return words
        if self.search(prefix):
            words.append(prefix)
            return words
        node = self.root
        for letter in prefix:
            node = node.data.get(letter)
        return _get_key(prefix, node)

    def print_data(self) -> None:
        def dfs(node, level):
            if not node:
                return
            for key, child in node.data.items():
                print("  " * level + key)
                dfs(child, level + 1)

        dfs(self.root, 0)

    def walk(self):
        def dfs(node):
            if not node:
                return {}
            children = {}
            for letter, child in node.data.items():
                children[letter] = dfs(child)
            return {
                letter: subtree if any(subtree.values()) else list(subtree)
                for letter, subtree in children.items()
            }

        return dfs(self.root)

    @classmethod
    def from_strings(cls, strings: list[str]) -> Trie:
        trie = cls()
        for s in strings:
            trie.insert(s)
        return trie



from __future__ import annotations
import json
from datetime import datetime
from dataclasses import dataclass, field
from tubeulator.utils.string_conv import to_camel_case
from pydantic import AliasGenerator, BaseModel, ConfigDict, PrivateAttr
from ..utils.paths import DtoEnum

class Mode(BaseModel):
    """
    Autogenerated from Line::Tfl.Api.Presentation.Entities.Mode
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    IsTflService: bool = None
    IsFarePaying: bool = None
    IsScheduledService: bool = None
    ModeName: str = None
    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Tfl')


ModeModel = Mode


class StatusSeverity(BaseModel):
    """
    Autogenerated from Line::Tfl.Api.Presentation.Entities.StatusSeverity
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    ModeName: str = None
    SeverityLevel: int = None
    Description: str = None
    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Tfl-2')


StatusSeverityModel = StatusSeverity


class PassengerFlow(BaseModel):
    """
    Autogenerated from Line::Tfl.Api.Presentation.Entities.PassengerFlow
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    TimeSlice: str = None
    Value: int = None
    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Tfl-3')


PassengerFlowModel = PassengerFlow


class TrainLoading(BaseModel):
    """
    Autogenerated from Line::Tfl.Api.Presentation.Entities.TrainLoading
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    Line: str = None
    LineDirection: str = None
    PlatformDirection: str = None
    Direction: str = None
    NaptanTo: str = None
    TimeSlice: str = None
    Value: int = None
    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Tfl-4')


TrainLoadingModel = TrainLoading


class Crowding(BaseModel):
    """
    Autogenerated from Line::Tfl.Api.Presentation.Entities.Crowding
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    PassengerFlows: list["PassengerFlowModel"]
    TrainLoadings: list["TrainLoadingModel"]
    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Tfl-5')


CrowdingModel = Crowding


class Identifier(BaseModel):
    """
    Autogenerated from Line::Tfl.Api.Presentation.Entities.Identifier
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    Id: str = None
    Name: str = None
    Uri: str = None
    FullName: str = None
    Type: str = None
    Crowding: CrowdingModel = None
    RouteType: str = None
    Status: str = None
    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Tfl-6')


IdentifierModel = Identifier


class LineGroup(BaseModel):
    """
    Autogenerated from Line::Tfl.Api.Presentation.Entities.LineGroup
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    NaptanIdReference: str = None
    StationAtcoCode: str = None
    LineIdentifier: list[str]
    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Tfl-7')


LineGroupModel = LineGroup


class LineModeGroup(BaseModel):
    """
    Autogenerated from Line::Tfl.Api.Presentation.Entities.LineModeGroup
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    ModeName: str = None
    LineIdentifier: list[str]
    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Tfl-8')


LineModeGroupModel = LineModeGroup


class AdditionalProperties(BaseModel):
    """
    Autogenerated from Line::Tfl.Api.Presentation.Entities.AdditionalProperties
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    Category: str = None
    Key: str = None
    SourceSystemKey: str = None
    Value: str = None
    Modified: datetime = None
    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Tfl-9')


AdditionalPropertiesModel = AdditionalProperties


class Place(BaseModel):
    """
    Autogenerated from Line::Tfl.Api.Presentation.Entities.Place
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    Id: str = None
    Url: str = None
    CommonName: str = None
    Distance: float = None
    PlaceType: str = None
    AdditionalProperties: list["AdditionalPropertiesModel"]
    Children: list["PlaceModel"]
    ChildrenUrls: list[str]
    Lat: float = None
    Lon: float = None
    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Tfl-10')


PlaceModel = Place


class StopPoint(BaseModel):
    """
    Autogenerated from Line::Tfl.Api.Presentation.Entities.StopPoint
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    NaptanId: str = None
    PlatformName: str = None
    Indicator: str = None
    StopLetter: str = None
    Modes: list[str]
    IcsCode: str = None
    SmsCode: str = None
    StopType: str = None
    StationNaptan: str = None
    AccessibilitySummary: str = None
    HubNaptanCode: str = None
    Lines: list["IdentifierModel"]
    LineGroup: list["LineGroupModel"]
    LineModeGroups: list["LineModeGroupModel"]
    FullName: str = None
    NaptanMode: str = None
    Status: bool = None
    Id: str = None
    Url: str = None
    CommonName: str = None
    Distance: float = None
    PlaceType: str = None
    AdditionalProperties: list["AdditionalPropertiesModel"]
    Children: list["PlaceModel"]
    ChildrenUrls: list[str]
    Lat: float = None
    Lon: float = None
    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Tfl-11')


StopPointModel = StopPoint


class RouteSectionNaptanEntrySequence(BaseModel):
    """
    Autogenerated from Line::Tfl.Api.Presentation.Entities.RouteSectionNaptanEntrySequence
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    Ordinal: int = None
    StopPoint: StopPointModel = None
    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Tfl-12')


RouteSectionNaptanEntrySequenceModel = RouteSectionNaptanEntrySequence


class RouteSection(BaseModel):
    """
    Autogenerated from Line::Tfl.Api.Presentation.Entities.RouteSection
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    Id: str = None
    LineId: str = None
    RouteCode: str = None
    Name: str = None
    LineString: str = None
    Direction: str = None
    OriginationName: str = None
    DestinationName: str = None
    ValidTo: datetime = None
    ValidFrom: datetime = None
    RouteSectionNaptanEntrySequence: list["RouteSectionNaptanEntrySequenceModel"]
    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Tfl-13')


RouteSectionModel = RouteSection


class Disruption(BaseModel):
    """
    Autogenerated from Line::Tfl.Api.Presentation.Entities.Disruption
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    Category: str = None
    Type: str = None
    CategoryDescription: str = None
    Description: str = None
    Summary: str = None
    AdditionalInfo: str = None
    Created: datetime = None
    LastUpdate: datetime = None
    AffectedRoutes: list["RouteSectionModel"]
    AffectedStops: list["StopPointModel"]
    ClosureText: str = None
    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Tfl-14')


DisruptionModel = Disruption


class ValidityPeriod(BaseModel):
    """
    Autogenerated from Line::Tfl.Api.Presentation.Entities.ValidityPeriod
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    FromDate: datetime = None
    ToDate: datetime = None
    IsNow: bool = None
    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Tfl-15')


ValidityPeriodModel = ValidityPeriod


class LineStatus(BaseModel):
    """
    Autogenerated from Line::Tfl.Api.Presentation.Entities.LineStatus
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    Id: int = None
    LineId: str = None
    StatusSeverity: int = None
    StatusSeverityDescription: str = None
    Reason: str = None
    Created: datetime = None
    Modified: datetime = None
    ValidityPeriods: list["ValidityPeriodModel"]
    Disruption: DisruptionModel = None
    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Tfl-16')


LineStatusModel = LineStatus


class MatchedRoute(BaseModel):
    """
    Autogenerated from Line::Tfl.Api.Presentation.Entities.MatchedRoute
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    RouteCode: str = None
    Name: str = None
    Direction: str = None
    OriginationName: str = None
    DestinationName: str = None
    Originator: str = None
    Destination: str = None
    ServiceType: str = None
    ValidTo: datetime = None
    ValidFrom: datetime = None
    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Tfl-17')


MatchedRouteModel = MatchedRoute


class LineServiceTypeInfo(BaseModel):
    """
    Autogenerated from Line::Tfl.Api.Presentation.Entities.LineServiceTypeInfo
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    Name: str = None
    Uri: str = None
    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Tfl-18')


LineServiceTypeInfoModel = LineServiceTypeInfo


class Line(BaseModel):
    """
    Autogenerated from Line::Tfl.Api.Presentation.Entities.Line
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    Id: str = None
    Name: str = None
    ModeName: str = None
    Disruptions: list["DisruptionModel"]
    Created: datetime = None
    Modified: datetime = None
    LineStatuses: list["LineStatusModel"]
    RouteSections: list["MatchedRouteModel"]
    ServiceTypes: list["LineServiceTypeInfoModel"]
    Crowding: CrowdingModel = None
    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Tfl-19')


LineModel = Line


class MatchedStop(BaseModel):
    """
    Autogenerated from Line::Tfl.Api.Presentation.Entities.MatchedStop
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    RouteId: int = None
    ParentId: str = None
    StationId: str = None
    IcsId: str = None
    TopMostParentId: str = None
    Direction: str = None
    Towards: str = None
    Modes: list[str]
    StopType: str = None
    StopLetter: str = None
    Zone: str = None
    AccessibilitySummary: str = None
    HasDisruption: bool = None
    Lines: list["IdentifierModel"]
    Status: bool = None
    Id: str = None
    Url: str = None
    Name: str = None
    Lat: float = None
    Lon: float = None
    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Tfl-20')


MatchedStopModel = MatchedStop


class StopPointSequence(BaseModel):
    """
    Autogenerated from Line::Tfl.Api.Presentation.Entities.StopPointSequence
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    LineId: str = None
    LineName: str = None
    Direction: str = None
    BranchId: int = None
    NextBranchIds: list[int]
    PrevBranchIds: list[int]
    StopPoint: list["MatchedStopModel"]
    ServiceType: str = None
    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Tfl-21')


StopPointSequenceModel = StopPointSequence


class OrderedRoute(BaseModel):
    """
    Autogenerated from Line::Tfl.Api.Presentation.Entities.OrderedRoute
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    Name: str = None
    NaptanIds: list[str]
    ServiceType: str = None
    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Tfl-22')


OrderedRouteModel = OrderedRoute


class RouteSequence(BaseModel):
    """
    Autogenerated from Line::Tfl.Api.Presentation.Entities.RouteSequence
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    LineId: str = None
    LineName: str = None
    Direction: str = None
    IsOutboundOnly: bool = None
    Mode: str = None
    LineStrings: list[str]
    Stations: list["MatchedStopModel"]
    StopPointSequences: list["StopPointSequenceModel"]
    OrderedLineRoutes: list["OrderedRouteModel"]
    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Tfl-23')


RouteSequenceModel = RouteSequence


class LineRouteSection(BaseModel):
    """
    Autogenerated from Line::Tfl.Api.Presentation.Entities.LineRouteSection
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    RouteId: int = None
    Direction: str = None
    Destination: str = None
    FromStation: str = None
    ToStation: str = None
    ServiceType: str = None
    VehicleDestinationText: str = None
    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Tfl-24')


LineRouteSectionModel = LineRouteSection


class MatchedRouteSections(BaseModel):
    """
    Autogenerated from Line::Tfl.Api.Presentation.Entities.MatchedRouteSections
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    Id: int = None
    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Tfl-25')


MatchedRouteSectionsModel = MatchedRouteSections


class RouteSearchMatch(BaseModel):
    """
    Autogenerated from Line::Tfl.Api.Presentation.Entities.RouteSearchMatch
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    LineId: str = None
    Mode: str = None
    LineName: str = None
    LineRouteSection: list["LineRouteSectionModel"]
    MatchedRouteSections: list["MatchedRouteSectionsModel"]
    MatchedStops: list["MatchedStopModel"]
    Id: str = None
    Url: str = None
    Name: str = None
    Lat: float = None
    Lon: float = None
    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Tfl-26')


RouteSearchMatchModel = RouteSearchMatch


class RouteSearchResponse(BaseModel):
    """
    Autogenerated from Line::Tfl.Api.Presentation.Entities.RouteSearchResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    Input: str = None
    SearchMatches: list["RouteSearchMatchModel"]
    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Tfl-27')


RouteSearchResponseModel = RouteSearchResponse


class Interval(BaseModel):
    """
    Autogenerated from Line::Tfl.Api.Presentation.Entities.Interval
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    StopId: str = None
    TimeToArrival: float = None
    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Tfl-28')


IntervalModel = Interval


class StationInterval(BaseModel):
    """
    Autogenerated from Line::Tfl.Api.Presentation.Entities.StationInterval
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    Id: str = None
    Intervals: list["IntervalModel"]
    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Tfl-29')


StationIntervalModel = StationInterval


class KnownJourney(BaseModel):
    """
    Autogenerated from Line::Tfl.Api.Presentation.Entities.KnownJourney
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    Hour: str = None
    Minute: str = None
    IntervalId: int = None
    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Tfl-30')


KnownJourneyModel = KnownJourney


class TwentyFourHourClockTime(BaseModel):
    """
    Autogenerated from Line::Tfl.Api.Presentation.Entities.TwentyFourHourClockTime
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    Hour: str = None
    Minute: str = None
    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Tfl-31')


TwentyFourHourClockTimeModel = TwentyFourHourClockTime


class ServiceFrequency(BaseModel):
    """
    Autogenerated from Line::Tfl.Api.Presentation.Entities.ServiceFrequency
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    LowestFrequency: float = None
    HighestFrequency: float = None
    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Tfl-32')


ServiceFrequencyModel = ServiceFrequency


class Period(BaseModel):
    """
    Autogenerated from Line::Tfl.Api.Presentation.Entities.Period
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    Type: str = None
    FromTime: TwentyFourHourClockTimeModel = None
    ToTime: TwentyFourHourClockTimeModel = None
    Frequency: ServiceFrequencyModel = None
    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Tfl-33')


PeriodModel = Period


class Schedule(BaseModel):
    """
    Autogenerated from Line::Tfl.Api.Presentation.Entities.Schedule
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    Name: str = None
    KnownJourneys: list["KnownJourneyModel"]
    FirstJourney: KnownJourneyModel = None
    LastJourney: KnownJourneyModel = None
    Periods: list["PeriodModel"]
    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Tfl-34')


ScheduleModel = Schedule


class TimetableRoute(BaseModel):
    """
    Autogenerated from Line::Tfl.Api.Presentation.Entities.TimetableRoute
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    StationIntervals: list["StationIntervalModel"]
    Schedules: list["ScheduleModel"]
    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Tfl-35')


TimetableRouteModel = TimetableRoute


class Timetable(BaseModel):
    """
    Autogenerated from Line::Tfl.Api.Presentation.Entities.Timetable
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    DepartureStopId: str = None
    Routes: list["TimetableRouteModel"]
    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Tfl-36')


TimetableModel = Timetable


class DisambiguationOption(BaseModel):
    """
    Autogenerated from Line::Tfl.Api.Presentation.Entities.Timetables.DisambiguationOption
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    Description: str = None
    Uri: str = None
    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Tfl-37')


DisambiguationOptionModel = DisambiguationOption


class Disambiguation(BaseModel):
    """
    Autogenerated from Line::Tfl.Api.Presentation.Entities.Timetables.Disambiguation
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    DisambiguationOptions: list["DisambiguationOptionModel"]
    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Tfl-38')


DisambiguationModel = Disambiguation


class TimetableResponse(BaseModel):
    """
    Autogenerated from Line::Tfl.Api.Presentation.Entities.TimetableResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    LineId: str = None
    LineName: str = None
    Direction: str = None
    PdfUrl: str = None
    Stations: list["MatchedStopModel"]
    Stops: list["MatchedStopModel"]
    Timetable: TimetableModel = None
    Disambiguation: DisambiguationModel = None
    StatusErrorMessage: str = None
    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Tfl-39')


TimetableResponseModel = TimetableResponse


class PredictionTiming(BaseModel):
    """
    Autogenerated from Line::Tfl.Api.Presentation.Entities.PredictionTiming
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    CountdownServerAdjustment: str = None
    Source: datetime = None
    Insert: datetime = None
    Read: datetime = None
    Sent: datetime = None
    Received: datetime = None
    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Tfl-40')


PredictionTimingModel = PredictionTiming


class Prediction(BaseModel):
    """
    Autogenerated from Line::Tfl.Api.Presentation.Entities.Prediction
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    Id: str = None
    OperationType: int = None
    VehicleId: str = None
    NaptanId: str = None
    StationName: str = None
    LineId: str = None
    LineName: str = None
    PlatformName: str = None
    Direction: str = None
    Bearing: str = None
    DestinationNaptanId: str = None
    DestinationName: str = None
    Timestamp: datetime = None
    TimeToStation: int = None
    CurrentLocation: str = None
    Towards: str = None
    ExpectedArrival: datetime = None
    TimeToLive: datetime = None
    ModeName: str = None
    Timing: PredictionTimingModel = None
    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Tfl-41')


PredictionModel = Prediction


class MetaModesGet200ApplicationJsonResponse(BaseModel):
    """
    Autogenerated from Line::MetaModesGet200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='MetaModesGet200ApplicationJsonResponse')


MetaModesGet200ApplicationJsonResponseModel = MetaModesGet200ApplicationJsonResponse


class Get200ApplicationJsonResponse(BaseModel):
    """
    Autogenerated from Line::Get200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Get200ApplicationJsonResponse')


Get200ApplicationJsonResponseModel = Get200ApplicationJsonResponse


class MetaSeverityGet200ApplicationJsonResponse(BaseModel):
    """
    Autogenerated from Line::MetaSeverityGet200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='MetaSeverityGet200ApplicationJsonResponse')


MetaSeverityGet200ApplicationJsonResponseModel = MetaSeverityGet200ApplicationJsonResponse


class MetaDisruptionCategories(BaseModel):
    """
    Autogenerated from Line::MetaDisruptionCategoriesGet200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='MetaDisruptionCategoriesGet200ApplicationJsonResponse')


MetaDisruptionCategoriesModel = MetaDisruptionCategories


class MetaServiceTypes(BaseModel):
    """
    Autogenerated from Line::MetaServiceTypesGet200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='MetaServiceTypesGet200ApplicationJsonResponse')


MetaServiceTypesModel = MetaServiceTypes


class idsGet200ApplicationJsonResponse(BaseModel):
    """
    Autogenerated from Line::ids-Get200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='ids-Get200ApplicationJsonResponse')


idsGet200ApplicationJsonResponseModel = idsGet200ApplicationJsonResponse


class ModemodesGet200ApplicationJsonResponse(BaseModel):
    """
    Autogenerated from Line::Mode-modes-Get200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Mode-modes-Get200ApplicationJsonResponse')


ModemodesGet200ApplicationJsonResponseModel = ModemodesGet200ApplicationJsonResponse


class RouteGet200ApplicationJsonResponse(BaseModel):
    """
    Autogenerated from Line::RouteGet200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='RouteGet200ApplicationJsonResponse')


RouteGet200ApplicationJsonResponseModel = RouteGet200ApplicationJsonResponse


class idsRouteGet200ApplicationJsonResponse(BaseModel):
    """
    Autogenerated from Line::ids-RouteGet200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='ids-RouteGet200ApplicationJsonResponse')


idsRouteGet200ApplicationJsonResponseModel = idsRouteGet200ApplicationJsonResponse


class ModemodesRouteGet200ApplicationJsonResponse(BaseModel):
    """
    Autogenerated from Line::Mode-modes-RouteGet200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Mode-modes-RouteGet200ApplicationJsonResponse')


ModemodesRouteGet200ApplicationJsonResponseModel = ModemodesRouteGet200ApplicationJsonResponse


class idsStatusstartDateToendDateGet200ApplicationJsonResponse(BaseModel):
    """
    Autogenerated from Line::ids-Status-startDate-To-endDate-Get200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='ids-Status-startDate-To-endDate-Get200ApplicationJsonResponse')


idsStatusstartDateToendDateGet200ApplicationJsonResponseModel = idsStatusstartDateToendDateGet200ApplicationJsonResponse


class idsStatusGet200ApplicationJsonResponse(BaseModel):
    """
    Autogenerated from Line::ids-StatusGet200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='ids-StatusGet200ApplicationJsonResponse')


idsStatusGet200ApplicationJsonResponseModel = idsStatusGet200ApplicationJsonResponse


class StatusseverityGet200ApplicationJsonResponse(BaseModel):
    """
    Autogenerated from Line::Status-severity-Get200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Status-severity-Get200ApplicationJsonResponse')


StatusseverityGet200ApplicationJsonResponseModel = StatusseverityGet200ApplicationJsonResponse


class ModemodesStatusGet200ApplicationJsonResponse(BaseModel):
    """
    Autogenerated from Line::Mode-modes-StatusGet200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Mode-modes-StatusGet200ApplicationJsonResponse')


ModemodesStatusGet200ApplicationJsonResponseModel = ModemodesStatusGet200ApplicationJsonResponse


class idStopPointsGet200ApplicationJsonResponse(BaseModel):
    """
    Autogenerated from Line::id-StopPointsGet200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='id-StopPointsGet200ApplicationJsonResponse')


idStopPointsGet200ApplicationJsonResponseModel = idStopPointsGet200ApplicationJsonResponse


class idsDisruptionGet200ApplicationJsonResponse(BaseModel):
    """
    Autogenerated from Line::ids-DisruptionGet200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='ids-DisruptionGet200ApplicationJsonResponse')


idsDisruptionGet200ApplicationJsonResponseModel = idsDisruptionGet200ApplicationJsonResponse


class ModemodesDisruptionGet200ApplicationJsonResponse(BaseModel):
    """
    Autogenerated from Line::Mode-modes-DisruptionGet200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='Mode-modes-DisruptionGet200ApplicationJsonResponse')


ModemodesDisruptionGet200ApplicationJsonResponseModel = ModemodesDisruptionGet200ApplicationJsonResponse


class idsArrivalsstopPointIdGet200ApplicationJsonResponse(BaseModel):
    """
    Autogenerated from Line::ids-Arrivals-stopPointId-Get200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='ids-Arrivals-stopPointId-Get200ApplicationJsonResponse')


idsArrivalsstopPointIdGet200ApplicationJsonResponseModel = idsArrivalsstopPointIdGet200ApplicationJsonResponse


class idsArrivalsGet200ApplicationJsonResponse(BaseModel):
    """
    Autogenerated from Line::ids-ArrivalsGet200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='Line')
    _component_schema_name: str = PrivateAttr(default='ids-ArrivalsGet200ApplicationJsonResponse')


idsArrivalsGet200ApplicationJsonResponseModel = idsArrivalsGet200ApplicationJsonResponse


class Deserialisers(DtoEnum):
    Tfl = Mode
    Tfl_2 = StatusSeverity
    Tfl_3 = PassengerFlow
    Tfl_4 = TrainLoading
    Tfl_5 = Crowding
    Tfl_6 = Identifier
    Tfl_7 = LineGroup
    Tfl_8 = LineModeGroup
    Tfl_9 = AdditionalProperties
    Tfl_10 = Place
    Tfl_11 = StopPoint
    Tfl_12 = RouteSectionNaptanEntrySequence
    Tfl_13 = RouteSection
    Tfl_14 = Disruption
    Tfl_15 = ValidityPeriod
    Tfl_16 = LineStatus
    Tfl_17 = MatchedRoute
    Tfl_18 = LineServiceTypeInfo
    Tfl_19 = Line
    Tfl_20 = MatchedStop
    Tfl_21 = StopPointSequence
    Tfl_22 = OrderedRoute
    Tfl_23 = RouteSequence
    Tfl_24 = LineRouteSection
    Tfl_25 = MatchedRouteSections
    Tfl_26 = RouteSearchMatch
    Tfl_27 = RouteSearchResponse
    Tfl_28 = Interval
    Tfl_29 = StationInterval
    Tfl_30 = KnownJourney
    Tfl_31 = TwentyFourHourClockTime
    Tfl_32 = ServiceFrequency
    Tfl_33 = Period
    Tfl_34 = Schedule
    Tfl_35 = TimetableRoute
    Tfl_36 = Timetable
    Tfl_37 = DisambiguationOption
    Tfl_38 = Disambiguation
    Tfl_39 = TimetableResponse
    Tfl_40 = PredictionTiming
    Tfl_41 = Prediction
    MetaModesGet200ApplicationJsonResponse = MetaModesGet200ApplicationJsonResponse
    Get200ApplicationJsonResponse = Get200ApplicationJsonResponse
    MetaSeverityGet200ApplicationJsonResponse = MetaSeverityGet200ApplicationJsonResponse
    MetaDisruptionCategoriesGet200ApplicationJsonResponse = MetaDisruptionCategories
    MetaServiceTypesGet200ApplicationJsonResponse = MetaServiceTypes
    ids_Get200ApplicationJsonResponse = idsGet200ApplicationJsonResponse
    Mode_modes_Get200ApplicationJsonResponse = ModemodesGet200ApplicationJsonResponse
    RouteGet200ApplicationJsonResponse = RouteGet200ApplicationJsonResponse
    ids_RouteGet200ApplicationJsonResponse = idsRouteGet200ApplicationJsonResponse
    Mode_modes_RouteGet200ApplicationJsonResponse = ModemodesRouteGet200ApplicationJsonResponse
    ids_Status_startDate_To_endDate_Get200ApplicationJsonResponse = idsStatusstartDateToendDateGet200ApplicationJsonResponse
    ids_StatusGet200ApplicationJsonResponse = idsStatusGet200ApplicationJsonResponse
    Status_severity_Get200ApplicationJsonResponse = StatusseverityGet200ApplicationJsonResponse
    Mode_modes_StatusGet200ApplicationJsonResponse = ModemodesStatusGet200ApplicationJsonResponse
    id_StopPointsGet200ApplicationJsonResponse = idStopPointsGet200ApplicationJsonResponse
    ids_DisruptionGet200ApplicationJsonResponse = idsDisruptionGet200ApplicationJsonResponse
    Mode_modes_DisruptionGet200ApplicationJsonResponse = ModemodesDisruptionGet200ApplicationJsonResponse
    ids_Arrivals_stopPointId_Get200ApplicationJsonResponse = idsArrivalsstopPointIdGet200ApplicationJsonResponse
    ids_ArrivalsGet200ApplicationJsonResponse = idsArrivalsGet200ApplicationJsonResponse


from __future__ import annotations
import json
from datetime import datetime
from dataclasses import dataclass, field
from tubeulator.utils.string_conv import to_camel_case
from pydantic import AliasGenerator, BaseModel, ConfigDict, PrivateAttr
from ..utils.paths import DtoEnum

class PredictionTiming(BaseModel):
    """
    Autogenerated from Vehicle::Tfl.Api.Presentation.Entities.PredictionTiming
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    CountdownServerAdjustment: str = None
    Source: datetime = None
    Insert: datetime = None
    Read: datetime = None
    Sent: datetime = None
    Received: datetime = None
    _source_schema_name: str = PrivateAttr(default='Vehicle')
    _component_schema_name: str = PrivateAttr(default='Tfl')


PredictionTimingModel = PredictionTiming


class Prediction(BaseModel):
    """
    Autogenerated from Vehicle::Tfl.Api.Presentation.Entities.Prediction
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    Id: str = None
    OperationType: int = None
    VehicleId: str = None
    NaptanId: str = None
    StationName: str = None
    LineId: str = None
    LineName: str = None
    PlatformName: str = None
    Direction: str = None
    Bearing: str = None
    DestinationNaptanId: str = None
    DestinationName: str = None
    Timestamp: datetime = None
    TimeToStation: int = None
    CurrentLocation: str = None
    Towards: str = None
    ExpectedArrival: datetime = None
    TimeToLive: datetime = None
    ModeName: str = None
    Timing: PredictionTimingModel = None
    _source_schema_name: str = PrivateAttr(default='Vehicle')
    _component_schema_name: str = PrivateAttr(default='Tfl-2')


PredictionModel = Prediction


class VehicleMatch(BaseModel):
    """
    Autogenerated from Vehicle::Tfl.Api.Presentation.Entities.VehicleMatch
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    Vrm: str = None
    Type: str = None
    Make: str = None
    Model: str = None
    Colour: str = None
    Compliance: str = None
    _source_schema_name: str = PrivateAttr(default='Vehicle')
    _component_schema_name: str = PrivateAttr(default='Tfl-3')


VehicleMatchModel = VehicleMatch


class idsArrivalsGet200ApplicationJsonResponse(BaseModel):
    """
    Autogenerated from Vehicle::ids-ArrivalsGet200ApplicationJsonResponse
    """

    model_config = ConfigDict(
        alias_generator=AliasGenerator(validation_alias=to_camel_case),
    )

    _source_schema_name: str = PrivateAttr(default='Vehicle')
    _component_schema_name: str = PrivateAttr(default='ids-ArrivalsGet200ApplicationJsonResponse')


idsArrivalsGet200ApplicationJsonResponseModel = idsArrivalsGet200ApplicationJsonResponse


class Deserialisers(DtoEnum):
    Tfl = PredictionTiming
    Tfl_2 = Prediction
    Tfl_3 = VehicleMatch
    ids_ArrivalsGet200ApplicationJsonResponse = idsArrivalsGet200ApplicationJsonResponse


from .types import RouteEnum


class BikePointEndpointRoutes(RouteEnum):
    ALL_LOCATIONS = "/"
    """Gets all bike point locations. The Place object has an addtionalProperties array
    which contains the nbBikes, nbDocks and nbSpaces numbers which give the status of
    the BikePoint. A mismatch in these numbers i.e. nbDocks - (nbBikes + nbSpaces) != 0
    indicates broken docks."""
    SEARCH_STATIONS = "/Search"
    """Search for bike stations by their name, a bike point's name often contains
    information about the name of the street or nearby landmarks, for example. Note that the
    search result does not contain the PlaceProperties i.e. the status or occupancy of the
    BikePoint, to get that information you should retrieve the BikePoint by its id on
    '/BikePoint/id."""
    BIKE_POINT_BY_ID = "/{id}"
    """Gets the bike point with the given id."""



from pprint import pprint

import defopt

from .codegen.gen import emit_deserialisers
from .codegen.populate import generate_schema_coverage
from .openapi.scan import count_namespace, scan_namespace


def deserialise(schema_name: str) -> None:
    """Map each API schema to a corresponding dataclass which deserialises its JSON."""
    deserialised = emit_deserialisers(schema_name=schema_name)
    print(deserialised)
    return


def populate() -> None:
    """Map all API schemas to corresponding dataclasses that deserialise their JSON.
    Write all generated Python to module files in the `tubeulator.generated` directory.
    """
    generate_schema_coverage()
    return


def list_schemas() -> None:
    """Print each schema name on a line (for processing in scripts etc over STDIN)."""
    for schema in sorted(count_namespace(ignore_responses=True), key=str.lower):
        print(schema)
    return


def namespace() -> None:
    """Make a namespace inventory"""
    ns = scan_namespace(ignore_responses=True)
    pprint(ns)
    return


def ns_count() -> None:
    """Count empty/non-empty in namespace inventory"""
    ns = count_namespace(ignore_responses=True)
    pprint(ns)
    return


def main():
    defopt.run(
        {
            "names": namespace,
            "count": ns_count,
            "deserialise": deserialise,
            "populate": populate,
            "schemas": list_schemas,
        },
        no_negated_flags=True,
        strict_kwonly=False,
        short={},
    )
    return
